% See:
% Baccala & Sameshima. Partial directed coherence: a new concept in neural
% structure determination. Biol. Cybern. 84:463-474, 2001.
%                http://dx.doi.org/10.1007/PL00007990
% 
% for the original Five-dimensional VAR[2] with loop and feedback Example.
% Here we are using a slightly 'm'odified version.

clear; clc; format compact; format short

disp('======================================================================');
disp('        Pentavariate linear VAR[2] Model Example 5 - closed-loop')
disp('  Modified from Baccala & Sameshima. Biol. Cybern. 84:463-474, 2001.')
disp('           x1==>x2  x1-->x3 x1-->x4 x4-->x5 x5-->x4 x5-->x2');
disp('  (***) Here the loop is close from x5-->x2, instead of x5-->x1. ');
disp('======================================================================');

ndiscard = 1000; % number of points discarded at beginning of series
ns = 2000;   % number of analyzed samples points
nd = ns + ndiscard;

%randn('state', 2.090663000000000e+005)
% To reproduce Figure 4.6 the following seed was used
randn('state', 3.090663000000000e+003)

wi = randn(5,nd);
x1 = zeros(1,nd); x2 = zeros(1,nd); x3 = zeros(1,nd); x4 = zeros(1,nd);
x5 = zeros(1,nd);
% Variables initialization
for t = 1:4,
   x1(t) = randn(1); x2(t) = randn(1); x3(t) = randn(1); x4(t) = randn(1);
   x5(t) = randn(1);
end;

%chLabels = []; % or 
chLabels = {'x_1';'x_2';'x_3';'x_4';'x_5'};

for t=5:nd,
   x1(t) = 0.95*sqrt(2)*x1(t-1) - 0.9025*x1(t-2)+ wi(1,t);
   x2(t) = -0.5*x1(t-1) + 0.5*x5(t-2) + wi(2,t); % In this example the loop is
   %                      ^^^^^^^^^^             % closed from x5 -> x2
   x3(t) = 0.4*x2(t-2) + wi(3,t);
   x4(t) = -0.5*x3(t-1) + 0.25*sqrt(2)*x4(t-1) + 0.25*sqrt(2)*x5(t-1) ...
           + wi(4,t);
   x5(t) = -0.25*sqrt(2)*x4(t-1) + 0.25*sqrt(2)*x5(t-1) + wi(5,t);
end;

y = [x1' x2' x3' x4' x5']; % data must be organized column-wise

u = y(nd - ns + 1:nd,:);

%==========================================================================
%             generalized PDCn estimation and analysis parameters
%==========================================================================

run example_analysis_parameters % Setting default paramaters 

run example_pre_processing      % Detrending and/or standardization

run example_mvar_estimation     % Estimating VAR and testing adequacy of 
                                % VAR model as well as performing GCT and ICT.

%=======Overriding some default parameters for plotting and analysis=======

flgPrinting = [1 1 1 0 0 0 1]; % overriding default setting

alpha=0.01;         % Significance level
% gct_signif = 0.05;  % Granger causality test significance level
% igct_signif = 0.05; % Instantaneous GCT significance level

metric = 'diag';

run example_pdc_analysis

%==========================================================================
%                        PDCn Matrix Layout Plotting
%==========================================================================
w_max=fs/2;
flgColor = 0;   
for kflgColor = flgColor,
   h=figure;
   set(h,'NumberTitle','off','MenuBar','none', ...
         'Name', 'Figure 4.6 gPDC - modified example from Baccal & Sameshima (2001)')
   [hxlabel hylabel] = xplot(c,flgPrinting,fs,w_max,chLabels,kflgColor, ...
                                              flgScale,flgMax,flgSignifColor);
   [ax,hT]=suplabel(['Linear model Example 4.3: ' ...
      int2str(ns) ' data points.'],'t');
   set(hT,'FontSize',12); % Title font size
   xplot_title(alpha,metric);
end;
disp('')
disp('==>Check & compare with Fig.4b, page 469 in Baccala & Sameshima (2001)')
disp(' ')
disp('==>In the original article the amplitude PDC has been plotted.')
disp('   Here we graph squared-PDC.')
disp(' ')
disp('======================================================================');
disp('  This completes the Example 4.3.');
disp('======================================================================');
